import os
import subprocess
import time
from pathlib import Path


from djangotools.cmdline import Command, Argument, CommandData

from djangotools.cmdline.commands.cron import CronCommand
from djangotools.config import get_settings
from djangotools.run import Server


class RunCommand(Command):

    NAME = "run"
    HELP = "Lance le serveur avec la config de production"
    ALIASES = "r"
    ARGUMENT = [
        Argument("-p", "--pid-file", help="Chemin du ficher PID" ),
        Argument("-o", "--stdout", nargs=1, help="Crée le server dans un autre processus"),
        Argument("-e", "--stderr", nargs=1, help="Crée le server dans un autre processus"),
        Argument("-b", "--bind",  help="Adresse d'écoute du serveur exemple localhost:8080", default="0.0.0.0:8000"),
        Argument("-C", "--no-cron", action="store_true", help="Ne pas lancer le cron"),

    ]

    def start_cron(self, hostname):
        self.cron = CronCommand.thread_run(CommandData(url=f"http://{hostname}"))


    def wait_cron_ends(self):
        self.cron.join()

    def run(self, data):
        settings = get_settings()
        if data.bind:
            host_name = data.bind
        else:
            host_name = settings.HOSTNAME

        if not data.no_cron:
            self.start_cron(host_name)

        pid_file = Path(data.pid_file or settings.CONFIG.pid_file)
        if not pid_file.parent.is_dir():
            pid_file.parent.mkdir(exist_ok=True, parents=True)
        pid_file.write_text(str(os.getpid()))

        server = Server()
        server.handle(addrport=host_name)

        if not data.no_cron:
            self.wait_cron_ends()