from pathlib import Path

from django.core.exceptions import ImproperlyConfigured


def get_module(data):
    if isinstance(data, str):
        data = __import__(data, fromlist=['object'])
    children = []
    path = Path(data.__file__)
    module_name = data.__name__
    if path.name == "__init__.py":
        path = path.parent
        for f in path.iterdir():
            if f.is_file() and f.name.lower().endswith(".py") and f.name != "__init__.py":
                children.append(f"{module_name}.{f.name[:-3]}")
            elif f.is_dir() and f.name  != "__pycache__":
                if (f / "__init__.py").is_file():
                    children.append(f"{module_name}.{f.name}")
    return data, children


def iter_module_content(roots, max_depth=None, element_type=None, classe=None, filter_attr=None, only_module=False):
    if isinstance(roots, str):
        roots = [roots]
    if classe is not None:
        element_type = type

    done = set()
    stack = [(x, 0) for x in roots]
    while stack:
        module, depth = stack.pop(0)
        if max_depth is not None and depth > max_depth: continue

        current, children = get_module(module)
        if current in done: continue

        if only_module:
            yield current
        elif filter_attr is None:
            for attr in dir(current):
                val = getattr(current, attr)
                try:
                    if ((element_type is None or isinstance(val, element_type)) and
                            (classe is None or issubclass(val, classe))):
                        yield attr, val
                except ImproperlyConfigured:
                    pass
        else:
            for attr in dir(current):
                val = getattr(current, attr)
                if filter_attr(current, attr, val):
                    yield attr, val



        stack.extend([ (x, depth+1) for x in children])
        done.add(current)

def iter_module(roots, max_depth=None):
    return iter_module_content(roots=roots, max_depth=max_depth, only_module=True)