import json
from pathlib import Path

from djangotools.common import models




class UrlPath:

    def __init__(self):
        self.liste = [self]

    def __truediv__(self, other):
        root = UrlPath()
        if isinstance(other, UrlPath):
            root.liste = self.liste + other.liste
        elif isinstance(other, str):
            root.liste = self.liste + [ConstPath(other)]

        return root

    def iter_models(self):
        for x in self.liste:
            if isinstance(x, (ModelPath, AttributePath, VariablePath)):
                yield x
        return

    def resolve(self):
        return [self]

    def get_wrapper(self, function):
        return

    def execute(self, manager):
        root = Path(".")
        for p in self.liste:
            x =  p.resolve(manager)
            if x is not None:
                root = root / x
        return root

    def copy(self):
        root = UrlPath()
        root.liste = list(self.liste)
        return root

    def get_path(self):
        liste = []
        for x in self.liste:
            liste.extend(x.resolve())
        self.liste=liste
        parts = [x.part for x in self.liste]
        return "/".join([ x for x in parts if x])

    @property
    def part(self):
        raise NotImplementedError()

    def as_string(self):
        return "/".join(str(x) for x in self.liste)

class ConstPath(UrlPath):
    """
    Une partie d'un patch contenant juste une string
    """
    def __init__(self, string):
        super().__init__()
        self.string = string


    @property
    def part(self):
        return self.string

    def __repr__(self):
        return self.string

class AttributePath(UrlPath):
    """
        Permet d'avancer dans le path, c'set bien l'objet self qui sera appelé et non ajouté en parametre
    """
    def __init__(self, model, attr):
        super().__init__()
        self.model = model
        self.attr = attr
        self.model_name = self.model if isinstance(self.model, str) else model.__name__
        self.key = f"{self.model_name.lower()}_{self.attr}"
        self.forward = None

    @property
    def part(self):
        return f"<str:{self.key}>"

    def __repr__(self):
        return f"{self.model_name}[{self.attr}]"


class ModelPath(UrlPath):
    """
        Permet d'avancer dans le path, c'set bien l'objet self qui sera appelé et non ajouté en parametre
        A utiliser ModelPath(Classe)[id]
    """

    def __init__(self, model):
        super().__init__()
        self.model = model

    def __getitem__(self, item):
        return self / AttributePath(self.model, item)

    @property
    def part(self):
        return None

    def __repr__(self):
        return  "{"+self.model.__name__+"}"

class DeferredModelPath(UrlPath):
    """
        Permet de faire comme un ModelPath avec un model qui n'est pas encore déclaré
    """
    def __init__(self, model):
        super().__init__()
        self.model = model
        self.attr = None


    def resolve(self):
        model = models.get_model(self.model)
        root = model._path_.resolve() if model._path_ else PointPath()

        attribute = AttributePath(model, model._key_)
        attribute.forward = self.attr
        ret =  ( root / ConstPath(model.__name__.lower()) / attribute).liste
        return ret

    def __getitem__(self, item):
        self.attr = item
        return self

    def __repr__(self):
        return  f"() => {self.model}[{self.attr}]"


class PointPath(UrlPath):
    """
        Un part qui ne change rien /a/. == /a/
    """
    @property
    def part(self):
        return None

    def __repr__(self):
        return "."


class VariablePath(UrlPath):
    """
    Permmet d'ajouter la variable en paramètre et non en self
    """

    def __init__(self, model, key, attr=None, resolve_var=None):
        super().__init__()
        self.model = models.get_model(model)
        self.attr = attr or self.model._key_
        self.key = key
        self._resolve_var = resolve_var

    def resolve_var(self, obj, kwargs):
        if self._resolve_var: return self._resolve_var(obj, kwargs)
        return self.default_resolve_var(obj, kwargs)

    def default_resolve_var(self, obj, kwargs):
        return self.model.objects.get(**{self.attr: kwargs[self.key]})

    @property
    def part(self):
        return f"<str:{self.key}>"
